package com.usky.gateway.service.config.mqtt;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.gateway.domain.EgDeviceConfig;
import com.usky.gateway.domain.GatewayDeviceConfig;
import com.usky.gateway.mapper.EgDeviceConfigMapper;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.core.MessageProducer;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;

import java.util.List;

/**
 * @author yq
 * @date 2021/11/1 16:37
 */
@ConditionalOnProperty(prefix = "mqtt", value = {"enabled"}, havingValue = "true")
@Configuration
public class MqttInConfig {

    @Autowired
    private MqttBaseConfig mqttBaseConfig;
    @Autowired
    private EgDeviceConfigMapper egDeviceConfigMapper;

    public static final String CHANNEL_NAME_INPUT = "mqttInputChannel";

    @Bean(name = CHANNEL_NAME_INPUT)
    public MessageChannel mqttInputChannel() {
        return new DirectChannel();
    }


    /**
     * 消息订阅绑定-消费者
     *
     * @return
     */
    @Bean
    public MessageProducer inbound() {
//        String[] tops = mqttBaseConfig.getMsgTopic().split(",");
        LambdaQueryWrapper<EgDeviceConfig> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(EgDeviceConfig::getDeviceUuid);
        List<EgDeviceConfig> list = egDeviceConfigMapper.selectList(queryWrapper);
        String[] tops = new String[list.size()];
        for (int i = 0; i < list.size(); i++) {
            tops[i] = "/"+list.get(i).getDeviceUuid()+"/control";
        }

        String clientId = "gateway-eg-kat-mqtt-in-" + System.currentTimeMillis();
        MqttPahoMessageDrivenChannelAdapter adapter = new MqttPahoMessageDrivenChannelAdapter(clientId,
                mqttBaseConfig.mqttClientFactory(), tops);
        adapter.setCompletionTimeout(mqttBaseConfig.getCompletionTimeout());
        adapter.setConverter(new DefaultPahoMessageConverter());
        adapter.setQos(2);
        adapter.setOutputChannel(mqttInputChannel());
        return adapter;
    }
}
