package com.usky.gateway.service.mqtt.control;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.gateway.domain.EgDeviceConfig;
import com.usky.gateway.domain.GatewayDeviceConfig;
import com.usky.gateway.mapper.EgDeviceConfigMapper;
import com.usky.gateway.mapper.GatewayDeviceConfigMapper;
import com.usky.gateway.service.config.mqtt.MqttOutConfig;
import com.usky.gateway.service.config.udp.UdpBaseConfig;
import com.usky.gateway.service.mqtt.MqttStrategy;
import com.usky.gateway.service.utils.ByteUtil;
import com.usky.gateway.service.utils.UdpUtil;
import com.usky.gateway.service.vo.MqttBaseVO;
import feign.form.util.CharsetUtil;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.util.Map;

@Service("control")
@Configuration
@Data
public class control implements MqttStrategy {

    @Autowired
    private UdpUtil udpUtil;
    @Autowired
    private ByteUtil byteUtil;
    @Resource
    private MqttOutConfig.MqttGateway mqttGateway;

    @Autowired
    private EgDeviceConfigMapper egDeviceConfigMapper;

    //处理下发命令消息，下发命令控制设备
    public String disposeMessage(MqttBaseVO mqttBaseVO) {
        String[] topics = mqttBaseVO.getTopic().split("/");
        String deviceUuid = topics[1];
        JSONObject obj_data = JSONObject.parseObject(mqttBaseVO.getData().toString());
        Integer commandId = Integer.parseInt(obj_data.get("id").toString());
        Object params = JSONObject.toJSONString(obj_data.get("params"));
        JSONObject params_data = JSON.parseObject(params.toString());
        String commandCode = params_data.getString("commandCode");
        String commandValue = params_data.getString("commandValue");

        //开门   17 40 00 00 E0 F4 4D 0D 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
        if(commandCode.equals("door_onoff")){
            //通过查询db文件获取对应udpIp udpPort 目标设备id 门号
            LambdaQueryWrapper<EgDeviceConfig> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.select(EgDeviceConfig::getDeviceId,EgDeviceConfig::getIp,EgDeviceConfig::getPort,EgDeviceConfig::getEgNumber)
                    .eq(EgDeviceConfig::getDeviceUuid,deviceUuid);
            EgDeviceConfig one = egDeviceConfigMapper.selectOne(queryWrapper);
            if(one != null){
                String devId = one.getDeviceId();
                byte[] deviceByte = byteUtil.toLH(Integer.parseInt(devId));
                int doorNum = Integer.parseInt(one.getEgNumber());
                byte bDoorNum = (byte)doorNum;
                byte[] requestBytes = new byte[] { 0x17, 0x40, 0x00, 0x00, deviceByte[0], deviceByte[1], deviceByte[2], deviceByte[3], bDoorNum, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
                byteUtil.printBytes(requestBytes);
                String responseData = udpUtil.sendUDPRequest(one.getIp(), Integer.parseInt(one.getPort()), requestBytes);

                //推送下发命令响应mqtt
                JSONObject jsonObject = new JSONObject();
                byte[] responseByte = responseData.getBytes(Charset.forName("ISO-8859-1"));  //"US-ASCII"
                byteUtil.printBytes(responseByte);

                if(responseByte[8] == 0x01){
                    jsonObject.put("result","success");

                }else{
                    JSONObject obj1 = new JSONObject();
                    obj1.put("code",-1);
                    obj1.put("message","open failed");
                    jsonObject.put("error",obj1);

                }

                jsonObject.put("timeStamp",System.currentTimeMillis());
                jsonObject.put("id",commandId);

                String res_topic = mqttBaseVO.getTopic().replace("control","controlResponse");
                mqttGateway.sendToMqtt(res_topic,jsonObject.toJSONString());
            }
        }


        return null;
    }


}
