package com.usky.hik.controller.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.usky.hik.service.utils.HttpClientUtils;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;


@RestController
@RequestMapping("/deviceAlarm")
@Slf4j
public class DeviceAlarmController {
    private static final Logger logger = LoggerFactory.getLogger(DeviceAlarmController.class);
    @Value("${alarm.url}")
    private String alarmUrl;

    @Autowired
    private HikThrowController hikThrowController;

    /**
     * 海康高空抛物事件回调接口
     * @param body
     * @return
     */
    @PostMapping("/skyThrow")
    public String skyThrow(@RequestBody String body) throws ParseException {
        System.out.println("body: "+body);

        JSONObject rec_body = JSON.parseObject(body);
        Object obj_params = JSON.toJSONString(rec_body.get("params"));
        JSONObject json_params = JSONObject.parseObject(obj_params.toString());

        JSONArray array_events = JSONArray.parseArray(json_params.get("events").toString());
        JSONObject json_events = JSONObject.parseObject(array_events.getJSONObject(0).toJSONString());

        //utc时间格式转换
        SimpleDateFormat fromDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
        SimpleDateFormat toDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String happenTime = json_events.get("happenTime").toString();
        Date date = fromDateFormat.parse(happenTime);
        Long dateTime = date.getTime();
        String toHappenTime = toDateFormat.format(dateTime);

        Object obj_data = JSON.toJSONString(json_events.get("data"));
        JSONObject json_data = JSONObject.parseObject(obj_data.toString());

        Object obj_target_attrs = JSON.toJSONString(json_data.get("targetAttrs"));
        JSONObject json_target_attrs = JSONObject.parseObject(obj_target_attrs.toString());
        String deviceIndexCode = json_target_attrs.get("cameraIndexCode").toString();
        JSONArray array_throw = JSONArray.parseArray(json_data.get("ObjectsThrownDetection").toString());
        JSONObject json_throw = JSONObject.parseObject(array_throw.getJSONObject(0).toJSONString());

        Object obj_image = JSON.toJSONString(json_throw.get("Image"));
        JSONObject json_image = JSONObject.parseObject(obj_image.toString());
        String resourcesContent = json_image.get("resourcesContent").toString();



        JSONObject jsonObject = new JSONObject();
        jsonObject.put("deviceId",deviceIndexCode);
        jsonObject.put("alarmTime",toHappenTime);
        jsonObject.put("sitePhoto",resourcesContent);

        jsonObject.put("alarmType","105");
        jsonObject.put("alarmObject","");
        jsonObject.put("alarmData","");
        jsonObject.put("alarmAttribute","高空抛物");
        jsonObject.put("alarmContent","高空抛物");
        jsonObject.put("alarmGrade",1);
        jsonObject.put("alarmAddress","");
        jsonObject.put("productCode","501_HGHK");

        String resultString = HttpClientUtils.doPostJson(alarmUrl,jsonObject.toJSONString());

        System.out.println("resultString: "+resultString);
        hikThrowController.insert(body,resultString);

        String responseBody = "HTTP/1.1 200 OK";
        return responseBody;
    }

}








