package com.usky.demo;


import com.usky.common.core.bean.ApiResult;
import com.usky.demo.domain.*;
import com.usky.demo.factory.RemoteTsdbProxyFallbackFactory;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(contextId = "remoteTsdbProxyService", value = "data-tsdb-proxy", fallbackFactory = RemoteTsdbProxyFallbackFactory.class)
public interface RemoteTsdbProxyService {

    /**
     * 设备实时数据查询
     */
    @PostMapping("/last")
    List<LastInnerResultVO> last(@RequestBody LastInnerQueryVO requestVO);

    /**
     * 单个设备实时数据查询
     * @param deviceUUId
     * @return
     */
    @GetMapping("/last")
    LastResultVO queryLastDeviceData(@RequestParam(value = "deviceUUId") String deviceUUId);

    /**
     * 批量设备实时数据查询
     * @param requestVO
     * @return
     */
    @PostMapping("/externalLast")
    List<LastResultVO> queryLastDeviceData(@RequestBody LastRequestVO requestVO);

    /**
     * 单个设备历史数据查询
     * @param deviceUUId
     * @param startTime
     * @param endTime
     * @return
     */
    @GetMapping("/history")
    HistoryResultVO queryHistoryDeviceData(@RequestParam(value = "deviceUUId") String deviceUUId,
                                                             @RequestParam(value = "startTime") String startTime,
                                                             @RequestParam(value = "endTime") String endTime);

    /**
     * 批量设备历史数据查询
     * @param requestVO
     * @return
     */
    @PostMapping("/history")
    List<HistoryResultVO> queryHistoryDeviceData(@RequestBody HistoryRequestVO requestVO);
}
